/*jshint -W082 */
'use strict';

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _htmlparser2 = require('htmlparser2');

var _htmlparser22 = _interopRequireDefault(_htmlparser2);

var _libApiJs = require('../lib/api.js');

exports['default'] = {
    /**
     * Parse html to PostHTML tree
     * @param  {String} html
     * @return {Object}
     */
    toTree: function toTree(html) {
        var bufArray = [],
            results = [];

        bufArray.last = function () {
            return this[this.length - 1];
        };

        var parser = new _htmlparser22['default'].Parser({
            onprocessinginstruction: function onprocessinginstruction(name, data) {
                name === '!doctype' && results.push('<' + data + '>');
            },
            oncomment: function oncomment(data) {
                var comment = '<!--' + data + '-->',
                    last = bufArray.last();

                if (!last) {
                    results.push(comment);
                    return;
                }

                last.content || (last.content = []);
                last.content.push(comment);
            },
            onopentag: function onopentag(tag, attrs) {
                var buf = {};

                buf.tag = tag;

                if (!isEmpty(attrs)) buf.attrs = attrs;

                if (isEmpty(buf)) buf = { content: '' };

                bufArray.push(buf);
            },
            onclosetag: function onclosetag() {
                var buf = bufArray.pop();

                if (bufArray.length === 0) {
                    results.push(buf);

                    return;
                }

                var last = bufArray.last();
                if (!(last.content instanceof Array)) {
                    last.content = [];
                }
                last.content.push(buf);
            },
            ontext: function ontext(text) {
                var last = bufArray.last();
                if (!last) {
                    results.push(text);
                    return;
                }

                last.content || (last.content = []);
                last.content.push(text);
            }
        });

        parser.write(html);
        parser.end();

        return results;
    },
    /**
     * Parse PostHTML tree to html
     * @param  {Object} tree    PostHTML tree
     * @param  {Object} options Parse options
     * @return {String}         html string
     */
    toHtml: function toHtml(tree) {
        var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

        /**
         * options parse
         * @param {Array}  singleTags           single tags array for extend default
         * @param {String} closingSingleTag     option for closing single tag
         *                                      Option:
         *                                          default: `<br>`
         *                                          slash: `<br/>`
         *                                          tag: `<br></br>`
         *
         */

        var _ref = tree.options || options;

        var optSingleTags = _ref.singleTags;
        var closingSingleTag = _ref.closingSingleTag;

        var SINGLE_TAGS = ['area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'menuitem', 'meta', 'param', 'source', 'track', 'wbr'];
        var singleTags = {};

        for (var i = 0, len = SINGLE_TAGS.length; i < len; i++) {
            singleTags[SINGLE_TAGS[i]] = 1;
        }

        if (optSingleTags) {
            for (var i = 0, len = optSingleTags.length; i < len; i++) {
                singleTags[optSingleTags[i]] = 1;
            }
        }

        return html(tree);

        function html(tree) {

            if (typeof tree === 'string') {
                return tree;
            }

            var buf = '';

            _libApiJs.walk.call([].concat(tree), function (node) {
                if (!node) return;
                if (typeof node === 'string') {
                    buf += node;
                    return;
                }
                var tag = node.tag || 'div';
                if (singleTags[tag]) {
                    buf += '<' + tag + attrs(node.attrs);
                    switch (closingSingleTag) {
                        case 'slash':
                            buf += ' />';
                            break;
                        case 'tag':
                            buf += '></' + tag + '>';
                            break;
                        default:
                            buf += '>';
                    }
                } else {
                    buf += '<' + tag + (node.attrs ? attrs(node.attrs) : '') + '>' + (node.content ? html(node.content) : '') + '</' + tag + '>';
                }
            });

            return buf;

            function attrs(obj) {
                var attr = '';
                for (var key in obj) {
                    attr += ' ' + key + (obj[key] && typeof obj[key] !== 'boolean' ? '="' + obj[key] + '"' : '');
                }
                return attr;
            }
        }
    }
};

function isEmpty(obj) {
    for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            return false;
        }
    }

    return true;
}
module.exports = exports['default'];